﻿#include <iostream>
#include <vector>
#include <iomanip>
#include <algorithm>

const size_t SCREEN_WIDTH = 80; // Максимальная ширина экрана
const size_t MAX_ASTERISK = SCREEN_WIDTH - 6; // Ограничение на длину столбца, с учетом увеличенного места для подписей

// Функция для ввода данных: количество чисел и сами числа
void input_data(std::vector<double>& numbers, size_t& bin_count) {
    size_t number_count;
    std::cerr << "Enter number count: ";
    std::cin >> number_count;

    numbers.resize(number_count);
    std::cerr << "Enter numbers: ";
    for (size_t i = 0; i < number_count; ++i) {
        std::cin >> numbers[i];
    }

    std::cerr << "Enter bin count: ";
    std::cin >> bin_count;
}

// Функция для расчета количества чисел в каждой корзине
void calculate_bins(const std::vector<double>& numbers, std::vector<size_t>& bins, double& min, double& max) {
    min = *std::min_element(numbers.begin(), numbers.end());
    max = *std::max_element(numbers.begin(), numbers.end());

    double bin_size = (max - min) / bins.size();

    for (const double& number : numbers) {
        size_t bin_index = static_cast<size_t>((number - min) / bin_size);
        if (bin_index >= bins.size()) bin_index = bins.size() - 1;
        bins[bin_index]++;
    }
}

// Функция для вывода гистограммы с промежуточными границами столбцов
void print_histogram(const std::vector<size_t>& bins, double min, double max) {
    size_t max_count = *std::max_element(bins.begin(), bins.end());
    bool needs_scaling = max_count > MAX_ASTERISK;
    double bin_size = (max - min) / bins.size();

    for (size_t i = 0; i < bins.size(); ++i) {
        // Вывод количества элементов с полем в 6 символов
        std::cout << std::setw(6) << std::right << bins[i] << "|";

        // Рассчитываем высоту столбца с учетом ограничения MAX_ASTERISK
        size_t height = needs_scaling ? static_cast<size_t>(MAX_ASTERISK * static_cast<double>(bins[i]) / max_count) : bins[i];

        // Выводим звездочки для визуализации количества элементов
        for (size_t j = 0; j < height; ++j) {
            std::cout << "*";
        }
        std::cout << "\n";

        // Выводим промежуточные границы между столбцами (исключая первую границу)
        if (i < bins.size() - 1) {
            double boundary = min + (i + 1) * bin_size;
            std::cout << std::fixed << std::setprecision(2) << boundary << "\n";
        }
    }
}

int main() {
    std::vector<double> numbers;
    size_t bin_count;

    input_data(numbers, bin_count);

    std::vector<size_t> bins(bin_count, 0);
    double min, max;
    calculate_bins(numbers, bins, min, max);
    print_histogram(bins, min, max);

    return 0;
}
